#include "Logging.h"
#include "Acd.h"
#include "AppDialogSetEndCommand.h"
#include <sstream>
#include "MIAQueue.h"

using namespace Dumais::JSON;

Acd::Acd(CallManager *callManager, AgentManager *agentManager , StaticConfiguration *config){
    this->mConfig = config;
    this->mCallManager = callManager;
    this->mAgentManager = agentManager;
    mLoginExtension = config->getValue("LOGIN_EXTENSION");
    mLogoutExtension = config->getValue("LOGOUT_EXTENSION");

    this->mCallManager->addObserver(this,mLoginExtension); // receive events for calls for the login extension
    this->mCallManager->addObserver(this,mLogoutExtension);// receive events for calls for the logout extension

    test=false;
}

Acd::~Acd(){
}


Queue* Acd::createQueue(std::string name)
{
    MIAQueue *pq = new MIAQueue(name,mCallManager);
    this->mQueueList[name]=pq;

    return pq;
}

Queue* Acd::getQueue(std::string name)
{
    if (mQueueList.find(name)!=mQueueList.end())
    {
        return mQueueList[name];
    }
    return 0;
}


void Acd::onRTPSessionSoundQueueEmpty(Call *call)
{
    //WARNING: This is called from the RTP worker Thread

}


void Acd::work()
{

//    this->mCallManager->redirectNonAnsweredCalls();
    for (QueueList::iterator it=mQueueList.begin();it!=mQueueList.end();it++)
    {
        Queue *q = it->second;
        q->routeOneCall(mAgentManager);
    }

}

void Acd::onIncommingCallAnswered(Call *call)
{

    Logging::log("Acd::onIncommingCallAnswered\r\n");
    const char* to = call->getOwner();
    if (!strcmp(to,mLogoutExtension.c_str()))
    {
        Agent *a = mAgentManager->getAgentWithDevice(call->getFrom());
        if (a)
        {
            a->logout();
        }
        mCallManager->release(call);
    } else if (!strcmp(to,mLoginExtension.c_str())) {
        call->getRTPSession()->play(mConfig->getValue("LOGIN_SOUND").c_str());
    }


}

void Acd::onIncommingCall(Call *call)
{
//    call->addRTPObserver(this);
    this->mCallManager->answerIncomming(call);
}


void Acd::onCallTerminated(Call *call)
{

    Logging::log("Acd::onCallTerminated\r\n");

}

void Acd::onCallAnswered(Call *call)
{
    

    Logging::log("Acd::onCallAnswered()\r\n");
    


}

void Acd::onRONA(Call *call)
{
}


void Acd::onCallFailed(Call *call)
{
    Logging::log("Acd::onCallFailed\r\n");
}

void Acd::onDTMF(Call *call,std::string digit)
{
    const char* to = call->getOwner();

    call->getRTPSession()->clearQueue();
    if (digit=="#" && !strcmp(to,mLoginExtension.c_str()))
    {
        std::string digits = call->getDigitQueue();
        digits.resize(digits.size()-1);
        Agent *agent =this->mAgentManager->getAgent(digits);
    
        if (agent)
        {
            agent->onLogin(call->getContact());
            Logging::log("Agent %s logged on\r\n",digits.c_str());
            mCallManager->release(call);
        } else {
            call->getRTPSession()->play(mConfig->getValue("ERROR_SOUND").c_str());
            
        }
        call->clearDigitQueue();
    }

}

void Acd::getQueuesStats(JSON& json)
{
    JSON j = json.addList("queues");
    for (QueueList::iterator it=mQueueList.begin();it!=mQueueList.end();it++)
    {
        Queue *q = it->second;
        q->toJSON(j);
        
    }

}

void Acd::getAgentsStats(Dumais::JSON::JSON& json)
{
    mAgentManager->toJSON(json);
}

