#include "Logging.h"
#include "Agent.h"
#include <limits.h>
#include "Queue.h"

using namespace resip;
using namespace Dumais::JSON;

Agent::Agent(std::string username){
    this->mUsername = username;
    mAgentState = LoggedOut;
    mAvailableTime = LONG_MAX;
//    mSubscribeExpire = expire;
}

Agent::~Agent()
{
}


std::string Agent::getAssociatedDevice()
{
    return mDevice.uri().user().data();
}

NameAddr Agent::getContact()
{
    return mDevice;
}

std::string Agent::getUserName()
{
    return this->mUsername;
}

void Agent::onAssignFailed()
{
    // If there was a failure to assign a call to an agent, we should unregister hum
    setAgentState(LoggedOut);    

    //The only way to get into Assign mode is to be Idle.
    // but we could have received a presence in the meantime and we would not be assigned anymore
/*    if (mAgentState==Assigning)
    {
        //Note the agent we be least idle time after this. This could be a problem
        setAgentState(Idle);
    }*/
}

void Agent::logout()
{
    if (mSubscriptionHandle.isValid())
    {
        Logging::log("ending subscription for %s\r\n",mUsername.c_str());
        mSubscriptionHandle->endCommand();
    }
}

bool Agent::isForDevice(resip::NameAddr device)
{
    std::string user1 = "";//"acd";
    const char* c = device.uri().user().data();
    user1 += c;

    std::string user2 = mDevice.uri().user().data();

    return (user1==user2);

}

void Agent::onLogin(resip::NameAddr device)
{
    this->mDevice = device;
    setAgentState(SubscribeExpired);    
}

void Agent::onSubscribeTerminated()
{
    setAgentState(LoggedOut);
}

void Agent::onAssigning()
{
    setAgentState(Assigning);
}

void Agent::onPresence(bool idle)
{
    if (idle)
    {
        setAgentState(Idle);
    } else {
        setAgentState(Unavailable);
    }
}

void Agent::setAgentState(ACDAgentState state)
{
    if (state == Idle)
    {
        time(&mAvailableTime);
    } else {
        mAvailableTime = LONG_MAX;
    }

    Logging::log("AgentState: %i\r\n",state);
    mAgentState = state;

}


ACDAgentState Agent::getAgentState()
{
    return mAgentState;
}

void Agent::setPhone(std::string phone)
{
    mPhone = phone;
}

std::string Agent::getPhone()
{
    return mPhone;
}

time_t Agent::getIdleTime()
{
    return this->mAvailableTime;
}

void Agent::setMemberOf(Queue *q)
{
    std::string qn = q->getName();
    mQueueList[qn]=qn;
}

bool Agent::memberOf(Queue *q)
{
    std::string qn = q->getName();
    return (mQueueList.find(qn)!=mQueueList.end());
}


void Agent::toJSON(Dumais::JSON::JSON& json)
{
    std::string state;
    std::string phone = mPhone;
    time_t t;
    time(&t);
    time_t idleTime = 0;
    switch (mAgentState)
    {
        case LoggedOut:
            state = "loggedout";
            phone = "";
        break;
        case SubscribeExpired:
            state = "subscribeexpired";
        break;
        case Subscribing:
            state = "subscribing";
        break;
        case Unavailable:
            state = "unavailable";
        break;
        case Idle:
            idleTime = t-mAvailableTime;
            state = "idle";
        break;
        case Assigning:
            state = "assigning";
        break;
    }

    JSON j = json.addObject();
    j.addValue(mUsername,"id");
    j.addValue(state,"state");
    j.addValue((int)idleTime,"idletime");
    j.addValue(phone,"device");
    JSON jQueues = j.addList("memberof");

    for (std::map<std::string,std::string>::iterator it= mQueueList.begin();it!=mQueueList.end();it++)
    {
        jQueues.addValue(it->second);
    }
}
