#ifndef AGENT_H
#define AGENT_H
#include <string>
#include <resip/stack/NameAddr.hxx>
#include <resip/dum/ClientSubscription.hxx>
#include <map>
#include "JSON.h"

class Queue;
enum ACDAgentState
{
    LoggedOut,
    SubscribeExpired,
    Subscribing,
    Unavailable,
    Idle,
    Assigning
};

class Agent{
private:
    friend class AgentManager;
    std::string mUsername;
    ACDAgentState mAgentState;
    time_t mAvailableTime;
    std::string mPhone;
    resip::NameAddr mDevice;
    //int mSubscribeExpire;
    std::map<std::string,std::string> mQueueList;
    resip::ClientSubscriptionHandle mSubscriptionHandle;
 
    void setAgentState(ACDAgentState state);


public:
	Agent(std::string username);
	~Agent();

    void setMemberOf(Queue *q);
    bool memberOf(Queue *q);

    void onLogin(resip::NameAddr device);
    void logout();
    void onSubscribeTerminated();
    void onAssigning();
    void onPresence(bool idle);
    void onAssignFailed();

    ACDAgentState getAgentState();
    bool isForDevice(resip::NameAddr device);

    void setPhone(std::string phone);
    std::string getPhone();
    time_t getIdleTime();
    std::string getUserName();
    resip::NameAddr getContact();
    std::string getAssociatedDevice();
    
    void toJSON(Dumais::JSON::JSON& json);

};

#endif

