#ifndef CALL_H
#define CALL_H

#include <resip/dum/AppDialogSet.hxx>
#include "RTPObserver.h"
#include "RTPSession.h"
#include "JSON.h"

class Agent;

class Call: public resip::AppDialogSet, public IRTPSessionObserver{
public:
    enum CallState
    {   
        Queued,
        Assigned,
        CallingAgent,
        Terminated
    };

	Call(resip::DialogUsageManager &dum);
	~Call();

    void addRTPObserver(RTPObserver *obs);

    void onSoundQueueEmpty();
    void setRTPSession(RTPSession *rtpSession);

    RTPSession* getRTPSession();

    bool isIncomming();
    void setIncomming(bool i);

    void onCallQueued();
    void onAssigned();
    void assignCall(Call *call, Agent *agent);
    void onAssignFailed(bool logout=true);
    void onTerminated();

    resip::DialogUsageManager& getDUM();
    CallState getCallState();
    std::string getDigitQueue();
    void clearDigitQueue();

    resip::NameAddr getFrom();
    resip::NameAddr getContact();
    const char* getOwner();
    
    std::string getID();

    void toJSON(Dumais::JSON::JSON &json);
    Call *getAssociatedQueuedCall();

private:
    friend class CallManager;

    resip::InviteSessionHandle mInviteSessionHandle;
    

    Agent *mAgent;
    RTPSession *mRtpSession;
    std::list<RTPObserver*> mRtpObservers;
    bool mIncomming;
    std::string mDigitQueue;
    resip::NameAddr mFrom;
    resip::NameAddr mOwner;
    resip::NameAddr mTo;
    resip::NameAddr mContact;
    Call *mQueuedCall;  //TODO: should use handle instead?
    Call *mAssignedCall;
    CallState mCallState;
    time_t mRingingTimeStamp;


};

#endif

