#include "ManagementConsole.h"
#include "Logging.h"

#include <sstream>
#include <sys/time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "HTTPRequest.h"
#include "JSON.h"

//TODO: This class is abit ugly. should separate sockets layer from http
using namespace Dumais::JSON;
void *consoleThreadStarter(void *p)
{
    ManagementConsole *m = (ManagementConsole*)p;
    m->run();
}


ManagementConsole::ManagementConsole(int port, std::string bindAddr, Acd *acd){
    mBindAddress = bindAddr;
    mPort = port;
    mAcd = acd;
}

ManagementConsole::~ManagementConsole(){
}

void ManagementConsole::start()
{
    Logging::log("Starting Management Console on %s:%i\r\n",mBindAddress.c_str(),mPort);
    pthread_create(&this->mThreadHandle, 0, consoleThreadStarter, (void*)this);
}

void ManagementConsole::stop()
{
    this->mQuit = true;
    pthread_join(this->mThreadHandle,0);
}

void ManagementConsole::run()
{
    mQuit=false;

    mServerSocket = socket(AF_INET, SOCK_STREAM, 0);
    setNonBlocking(mServerSocket);
    struct hostent* host;
    host = gethostbyname(mBindAddress.c_str());
    int on=1;
    setsockopt(mServerSocket, SOL_SOCKET, SO_REUSEADDR, &on,sizeof(on));
    sockaddr_in sockadd;
    sockadd.sin_family=AF_INET;
    sockadd.sin_addr.s_addr=*((unsigned long*) host->h_addr);
    sockadd.sin_port=htons(mPort);

    if (bind(mServerSocket,(struct sockaddr *)&sockadd,sizeof(sockadd))<0)
    {
        Logging::log("could not bind console socket\r\n");
        return;
    }

    listen(mServerSocket,MAXCLIENTS);
    fd_set socks;
    struct timeval timeout;

    for (int i=0;i<MAXCLIENTS;i++) mClients[i]=0;
    int numSocks=0;
    int largestFD=0;
    while (!mQuit)
    {
        FD_ZERO(&socks);
        FD_SET(mServerSocket,&socks);
        largestFD=mServerSocket;
        for (int i=0;i<MAXCLIENTS;i++)
        {
            if (mClients[i]!=0){
                FD_SET(mClients[i],&socks);    
                if (mClients[i]>largestFD) largestFD=mClients[i];
            }
        }    
    
        timeout.tv_sec = 1;
        timeout.tv_usec = 0;
        numSocks = select(largestFD+1, &socks, 0, 0, &timeout);
        if (numSocks>0)
        {
            if (FD_ISSET(mServerSocket,&socks))
            {
                acceptClient();
            }

            for (int i=0;i<MAXCLIENTS;i++)
            {
                if (mClients[i]!=0)
                {
                    if (FD_ISSET(mClients[i],&socks))
                    {
                        readClient(i);
                    }
                }
            }

        }
        //select to read client and listen sockets
    }
}

void ManagementConsole::acceptClient()
{
    int i=0;    
    while (i<MAXCLIENTS)
    {
        if (mClients[i]==0)
        {
            mClients[i] = accept(mServerSocket,0,0);
            setNonBlocking(mClients[i]);
            return;
        }
        i++;
    }
}


void ManagementConsole::readClient(int index)
{
    int sock = mClients[index];
    char buf[2000];
    int size=1;
    while (size>0)
    {
        size = read(sock,(char*)&buf,2000);
        if (size>0)
        {
            std::string request = (char*)&buf;
            processRequest(request,index);
        } else if (size==0){
            Logging::log("Closing client console connection\r\n");
            close(sock);
            mClients[index]=0;
        }
    }

}

void ManagementConsole::processRequest(std::string request, int index)
{
    int sock = mClients[index];
    std::string stat = "";
    HTTPRequest req(request);
    JSON json;
    if (req.getRequest()=="agents"){
        mAcd->getAgentsStats(json);
    } else if (req.getRequest()=="queues"){
        mAcd->getQueuesStats(json);
    }

    stat = json.stringify(false);
    if (stat!="{invalid}"){
        std::stringstream message;
        message << "HTTP/1.1 200 OK\r\nContent-Type: application/json;charset=utf-8\r\nContent-Length: ";
        message << stat.size() << "\r\n\r\n" << stat;
        send(sock,message.str().c_str(),message.str().size(),0);
        close(sock);
        mClients[index]=0;
    }

}

void ManagementConsole::setNonBlocking(int sock)
{
    int tmp = fcntl(sock,F_GETFL);
    fcntl(sock,F_SETFL,tmp|O_NONBLOCK);
    return;
}
