#include "Logging.h"
#include "Queue.h"
#include "AppDialogSetPeriodicAnnouncementCommand.h"

using namespace Dumais::JSON;

Queue::Queue(std::string queueName, 
    CallManager* callManager)
{
    mQueueName = queueName;
    mCallManager = callManager;

    this->mCallManager->addObserver(this,queueName);


}

Queue::~Queue(){
}

std::string Queue::getName()
{
    return mQueueName;
}

void Queue::onCallTerminated(Call *call)
{
    //TODO: It is possible that, after a transfer, the agent hangs up between the REFER and the INVITE. In that
    //      case, we could detect this with the final NOTIFY that the caller would send us. We should
    //      queue the call back and assign another agent to it. The current code would leave the caller
    //      in the limbo
    Logging::log("Removing call (state=%i) from queue %s\r\n",call->getCallState(),mQueueName.c_str());
    call->onTerminated();
    mCallList.remove(call->getHandle());
}

void Queue::onCallAnswered(Call *call)
{
    Logging::log("Acd::onCallAnswered()\r\n");

    mCallManager->transfer(call->getAssociatedQueuedCall(),call);

}

void Queue::onCallFailed(Call *call)
{
    call->onAssignFailed();
    Logging::log("Call failed on queue %s. Putting call back into queue\r\n",mQueueName.c_str());
}

void Queue::onRONA(Call *call)
{
    call->onAssignFailed(false);
    Logging::log("Call failed on queue %s. Putting call back into queue\r\n",mQueueName.c_str());
}


void Queue::onIncommingCallAnswered(Call *call)
{
    RTPSession *rtpSession = call->getRTPSession();
    rtpSession->setMusic(mQueueMusic.c_str());
    rtpSession->play(this->mWelcome.c_str());
    AppDialogSetPeriodicAnnouncementCommand cmd(this->mAnnouncement,this->mAnnouncementInterval, call->getHandle()); 
    call->getDUM().getSipStack().post(cmd,this->mAnnouncementInterval,&call->getDUM());
}

void Queue::onIncommingCall(Call *call)
{
    Logging::log("Incomming call queued on %s\r\n",mQueueName.c_str());
    mCallManager->answerIncomming(call);
    call->onCallQueued();
    mCallList.push_back(call->getHandle());
}

Call* Queue::getFirstCall()
{
    for (std::list<resip::AppDialogSetHandle>::iterator it= mCallList.begin();it!=mCallList.end();it++)
    {
        Call *call = dynamic_cast<Call*>((*it).get());
        if (call)
        {
            if (call->getCallState()==Call::Queued)
            {
                return call;
            }
        } else {
            mCallList.erase(it);
        }
    }
    return 0; 
}

void Queue::onDTMF(Call *call,std::string digit)
{
    if (digit=="0")
    {
        mCallManager->blindTransfer(call,"bailout");
    }
}

void Queue::setAnnouncement(std::string music, std::string welcome, std::string announce, int interval)
{
    this->mQueueMusic = music;
    this->mWelcome = welcome;
    this->mAnnouncement = announce;
    this->mAnnouncementInterval = interval;
}

void Queue::toJSON(Dumais::JSON::JSON& json)
{
    JSON j = json.addObject();
    j.addValue(getName(),"name");
    j.addValue(this->mQueueMusic,"music");
    j.addValue(this->mWelcome,"welcome");
    j.addValue(this->mAnnouncement,"announcement");
    j.addValue(this->mAnnouncementInterval,"accouncement-interval");

    JSON jCalls = j.addList("calls");
    for (std::list<resip::AppDialogSetHandle>::iterator it= mCallList.begin();it!=mCallList.end();it++)
    {
        Call *call = dynamic_cast<Call*>((*it).get());
        if (call){
            call->toJSON(jCalls);
        }
    }
}
