#include "Logging.h"
#include "UserAgent.h"
#include "Acd.h"
#include "AgentManager.h"
#include "StaticConfiguration.h"
#include "DynamicConfiguration.h"
#include "ManagementConsole.h"

int main(int argc, char **argv)
{
    std::string staticConfigFile;
    std::string queueConfigFile;
    bool daemon = false;
    
    for (int i=0;i<argc;i++)
    {
        std::string param = argv[i];
        if (param=="-s" && argc>(i+1))
        {
            staticConfigFile = argv[i+1];
        } else if (param=="-q" && argc>(i+1))
        {
            queueConfigFile = argv[i+1];
        } else if (param=="-d"){
            daemon=true;
        }
    }

    if (staticConfigFile== "")
    {
        printf("Please provide a config file\r\n");
        exit(-1);
    }

    StaticConfiguration config(staticConfigFile);
    Logging::log("Starting ACD Server\r\n");
    if (daemon)
    {
        //TODO: should tell logging facility to stop output
        if (fork()>0) exit(0);
         
    }

    AccountSettings account;
    std::string localIP=config.getValue("LISTENADDRESS");

    account.mUserName = config.getValue("ACCOUNT_USERNAME");
    account.mPin = config.getValue("ACCOUNT_PASSWORD");
    account.mProxy = config.getValue("ACCOUNT_PBX_IP");
    account.mDisplayName = config.getValue("ACCOUNT_DISPLAY_NAME");
    account.mPort = atoi(config.getValue("LISTENPORT").c_str());
    UserAgent ua(account,config.getValue("ACCOUNT_USERAGENT"));
    ua.registerAccount();

    RTPSessionFactory rtpSessionFactory(atoi(config.getValue("LOWPORT").c_str()),atoi(config.getValue("HIGHPORT").c_str()));
    rtpSessionFactory.setLocalIP(localIP);
    CallManager callManager(&ua,&rtpSessionFactory);
    callManager.setRONATimeout(atoi(config.getValue("RONA").c_str()));
    AgentManager agentManager(&ua,atoi(config.getValue("SUBSCRIBE_EXPIRE").c_str()));
    Acd acd(&callManager,&agentManager,&config);

    // Load queue config now that the engine is up
    DynamicConfiguration dconfig(&agentManager,&acd);
    dconfig.loadScript(queueConfigFile);

    ManagementConsole console(atoi(config.getValue("CONSOLEPORT").c_str()),config.getValue("CONSOLE_BIND"),&acd);
    console.start();
    while (1){
        ua.work();
        acd.work();
        agentManager.work();
    }
    console.stop();

    //TODO: Cleanup
}
